#include "TAlSAMURAIRawDataExample.hh"

#include <iostream>

#include "TArtAnaFile.hh"
#include "TArtAnaClock.hh"

#include "EAnalyser.hh"
#include "EWNum.hh"

#include "TArtCore.hh"
#include "TArtStoreManager.hh"
#include "segidlist.hh"

TAlSAMURAIRawDataExample::TAlSAMURAIRawDataExample() : TArtAnaLoop()
{;}

TAlSAMURAIRawDataExample::~TAlSAMURAIRawDataExample()
{
}

void TAlSAMURAIRawDataExample::Construct()
{

  tree      = new TTree("tree","samurai rawdata tree");
  tree->Branch("eventn",&eventnum,"eventn/I");
  tree->Branch("sca39",&efn39sca,"sca39/I");
  tree->Branch("sca40",&efn40sca,"sca40/I");
  tree->Branch("sca41",&efn41sca,"sca41/I");
  tree->Branch("sca42",&efn42sca,"sca42/I");
  tree->Branch("sca43",&efn43sca,"sca43/I");
  tree->Branch("sca44",&efn44sca,"sca44/I");
  tree->Branch("sca46",&efn46sca,"sca46/I");

  h1        = new TH2F("sca39_40", "sca39_40correlation", 1000, 0, 20000000, 1000, 0, 20000000);
  h2        = new TH2F("sca39_41", "sca39_41correlation", 1000, 0, 20000000, 1000, 0, 20000000);
  h3        = new TH2F("sca39_42", "sca39_42correlation", 1000, 0, 20000000, 1000, 0, 20000000);
  h4        = new TH2F("sca39_43", "sca39_43correlation", 1000, 0, 20000000, 1000, 0, 20000000);
  h5        = new TH2F("sca39_44", "sca39_44correlation", 1000, 0, 20000000, 1000, 0, 20000000);
  h6        = new TH2F("sca39_46", "sca39_46correlation", 1000, 0, 20000000, 1000, 0, 20000000);
  s1        = new TH1F("sca39_40sub", "sca39_40subtraction", 1000, -2000000, 2000000);
  s2        = new TH1F("sca39_41sub", "sca39_41subtraction", 1000, -20000000000, 20000000000);
  s3        = new TH1F("sca39_42sub", "sca39_42subtraction", 1000, -2000000, 2000000);
  s4        = new TH1F("sca39_43sub", "sca39_43subtraction", 1000, -2000000, 2000000);
  s5        = new TH1F("sca39_44sub", "sca39_44subtraction", 1000, -2000000, 2000000);
  s6        = new TH1F("sca39_46sub", "sca39_46subtraction", 1000, -200000000, 200000000);
}

void TAlSAMURAIRawDataExample::Calculate()
{
  TArtStoreManager * sman = TArtStoreManager::Instance();
  TArtRawEventObject *rawevent = (TArtRawEventObject *)sman->FindDataContainer("RawEvent");

  using namespace Analyser;
  using namespace WNum::RAWDATA;  

  int id=0;
  eventnum = rawevent->GetEventNumber();
  efn39sca = 0;
  efn40sca = 0;
  efn41sca = 0;
  efn42sca = 0;
  efn43sca = 0;
  efn44sca = 0;
  efn46sca = 0;

  for(int i=0;i<rawevent->GetNumSeg();i++){

    TArtRawSegmentObject *seg = rawevent->GetSegment(i);
    int efn = (int)seg->GetAddress();
    int device = (int)seg->GetDevice();
    int fp = (int)seg->GetFP();
    int detector = (int)seg->GetDetector();
    int module = (int)seg->GetModule();

    for(int j=0;j<seg->GetNumData();j++){

      TArtRawDataObject *d = seg->GetData(j);
      int geo = (int)d->GetGeo(); 
      int ch = (int)d->GetCh();
      int val = (int)d->GetVal(); 
      int edge = (int)d->GetEdge(); 

      if(efn==39&&detector==63&&ch==0){
	if(efn39sca == 0)
	  efn39sca = val;
	else TArtCore::Info(__FILE__,"funny 39 scaler");
      }
      if(efn==40&&detector==63&&ch==0){
	if(efn40sca == 0)
	  efn40sca = val;
	else TArtCore::Info(__FILE__,"funny 40 scaler");
      }
      if(efn==41&&detector==63&&ch==0){
	if(efn41sca == 0)
	  efn41sca = val;
	else TArtCore::Info(__FILE__,"funny 41 scaler");
      }

      if(efn==42&&detector==63&&ch==0){
	if(efn42sca == 0)
	  efn42sca = val;
	else TArtCore::Info(__FILE__,"funny 42 scaler");
      }
      if(efn==43&&detector==63&&ch==0){
	if(efn43sca == 0)
	  efn43sca = val;
	else TArtCore::Info(__FILE__,"funny 43 scaler");
      }
      if(efn==44&&detector==63&&ch==0){
	if(efn44sca == 0)
	  efn44sca = val;
	else TArtCore::Info(__FILE__,"funny 44 scaler");
      }
      if(efn==46&&detector==63&&ch==0){
	if(efn46sca == 0)
	  efn46sca = val;
	else TArtCore::Info(__FILE__,"funny 46 scaler");
      }

    }

  }

  tree->Fill();

  h1->Fill(efn39sca,efn40sca);
  h2->Fill(efn39sca,efn41sca);
  h3->Fill(efn39sca,efn42sca);
  h4->Fill(efn39sca,efn43sca);
  h5->Fill(efn39sca,efn44sca);
  h6->Fill(efn39sca,efn46sca);
  s1->Fill(efn39sca-efn40sca);
  s2->Fill(efn39sca-efn41sca);
  s3->Fill(efn39sca-efn42sca);
  s4->Fill(efn39sca-efn43sca);
  s5->Fill(efn39sca-efn44sca);
  s6->Fill(efn39sca-efn46sca);
  

}

void TAlSAMURAIRawDataExample::Destruct()
{
}

const char* TAlSAMURAIRawDataExample::ClassName() const
{
  return "TAlSAMURAIRawDataExample";
}
